/* $Id: algotest.c,v 1.2 2002/01/25 06:51:02 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rlcrypto/algorithms.h>

const struct test_st
{
  enum { NONE, ENC_DEC, ENCRYPT, DECRYPT, DIGEST } kind;
  const SYM_METHOD *(*sym)(void);
  const MODE_METHOD *(*mode)(void);
#if 0
  const DIGEST_METHOD *(*digest)(void);
#endif
  size_t key_bits;
  size_t keylen; unsigned char key[64];
  size_t ivlen; unsigned char iv[64];
  int iterations;
  size_t inputlen; unsigned char input[128];
  size_t outputlen; unsigned char output[128];
} tests[] =
  {
    /* The following test values come from appendix C of FIPS-197 */
    {
      ENC_DEC,
      &AES, 0, 128,
      16, { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
	    0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F },
      0, { 0 },
      1,
      16, { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
	    0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF },
      16, { 0x69, 0xC4, 0xE0, 0xD8, 0x6A, 0x7B, 0x04, 0x30,
	    0xD8, 0xCD, 0xB7, 0x80, 0x70, 0xB4, 0xC5, 0x5A }
    },
    {
      ENC_DEC,
      &AES, 0, 192,
      24, { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
	    0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
	    0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17 },
      0, { 0 },
      1,
      16, { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
	    0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF },
      16, { 0xDD, 0xA9, 0x7C, 0xA4, 0x86, 0x4C, 0xDF, 0xE0,
	    0x6E, 0xAF, 0x70, 0xA0, 0xEC, 0x0D, 0x71, 0x91 }
    },
    {
      ENC_DEC,
      &AES, 0, 256,
      32, { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
	    0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
	    0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17,
	    0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F },
      0, { 0 },
      1,
      16, { 0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77,
	    0x88, 0x99, 0xAA, 0xBB, 0xCC, 0xDD, 0xEE, 0xFF },
      16, { 0x8E, 0xA2, 0xB7, 0xCA, 0x51, 0x67, 0x45, 0xBF,
	    0xEA, 0xFC, 0x49, 0x90, 0x4B, 0x49, 0x60, 0x89 }
    },

    /* The following values are the NIST test vectors, done over 1000 iters. */
    {
      ENC_DEC,
      &AES, 0, 128,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      0, { 0 },
      10000,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0xC3, 0x4C, 0x05, 0x2C, 0xC0, 0xDA, 0x8D, 0x73,
	    0x45, 0x1A, 0xFE, 0x5F, 0x03, 0xBE, 0x29, 0x7F }
    },
    {
      ENC_DEC,
      &AES, 0, 192,
      24, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	    0, 0, 0, 0, 0, 0, 0, 0 },
      0, { 0 },
      10000,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0xF3, 0xF6, 0x75, 0x2A, 0xE8, 0xD7, 0x83, 0x11,
	    0x38, 0xF0, 0x41, 0x56, 0x06, 0x31, 0xB1, 0x14 }
    },
    {
      ENC_DEC,
      &AES, 0, 256,
      32, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      0, { 0 },
      10000,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0x8B, 0x79, 0xEE, 0xCC, 0x93, 0xA0, 0xEE, 0x5D,
	    0xFF, 0x30, 0xB4, 0xEA, 0x21, 0x63, 0x6D, 0xA4 }
    },
    {
      ENCRYPT,
      &AES, &CBC, 128,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      10000,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0x8A, 0x05, 0xFC, 0x5E, 0x09, 0x5A, 0xF4, 0x84,
	    0x8A, 0x08, 0xD3, 0x28, 0xD3, 0x68, 0x8E, 0x3D }
    },
    {
      ENCRYPT,
      &AES, &CBC, 192,
      24, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	    0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      10000,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0x7B, 0xD9, 0x66, 0xD5, 0x3A, 0xD8, 0xC1, 0xBB,
	    0x85, 0xD2, 0xAD, 0xFA, 0xE8, 0x7B, 0xB1, 0x04 }
    },
    {
      ENCRYPT,
      &AES, &CBC, 256,
      32, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      10000,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0xFE, 0x3C, 0x53, 0x65, 0x3E, 0x2F, 0x45, 0xB5,
	    0x6F, 0xCD, 0x88, 0xB2, 0xCC, 0x89, 0x8F, 0xF0 }
    },
    {
      DECRYPT,
      &AES, &CBC, 128,
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      10000,
      16, { 0xFA, 0xCA, 0x37, 0xE0, 0xB0, 0xC8, 0x53, 0x73,
	    0xDF, 0x70, 0x6E, 0x73, 0xF7, 0xC9, 0xAF, 0x86 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    {
      DECRYPT,
      &AES, &CBC, 192,
      24, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	    0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      10000,
      16, { 0x5D, 0xF6, 0x78, 0xDD, 0x17, 0xBA, 0x4E, 0x75,
	    0xB6, 0x17, 0x68, 0xC6, 0xAD, 0xEF, 0x7C, 0x7B },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    {
      DECRYPT,
      &AES, &CBC, 256,
      32, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
      10000,
      16, { 0x48, 0x04, 0xE1, 0x81, 0x8F, 0xE6, 0x29, 0x75,
	    0x19, 0xA3, 0xE8, 0x8C, 0x57, 0x31, 0x04, 0x13 },
      16, { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
    },
    
    /* Blowfish test vectors, taken from Counterpane */
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      0, { 0 },
      1,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      8, { 0x4E, 0xF9, 0x97, 0x45, 0x61, 0x98, 0xDD, 0x78 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF },
      0, { 0 },
      1,
      8, { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF },
      8, { 0x51, 0x86, 0x6F, 0xD5, 0xB8, 0x5E, 0xCB, 0x8A }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      0, { 0 },
      1,
      8, { 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01 },
      8, { 0x7D, 0x85, 0x6F, 0x9A, 0x61, 0x30, 0x63, 0xF2 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11 },
      0, { 0 },
      1,
      8, { 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11 },
      8, { 0x24, 0x66, 0xDD, 0x87, 0x8B, 0x96, 0x3C, 0x9D }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      0, { 0 },
      1,
      8, { 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11 },
      8, { 0x61, 0xF9, 0xC3, 0x80, 0x22, 0x81, 0xB0, 0x96 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11 },
      0, { 0 },
      1,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      8, { 0x7D, 0x0C, 0xC6, 0x30, 0xAF, 0xDA, 0x1E, 0xC7 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      0, { 0 },
      1,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      8, { 0x4E, 0xF9, 0x97, 0x45, 0x61, 0x98, 0xDD, 0x78 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      0, { 0 },
      1,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      8, { 0x0A, 0xCE, 0xAB, 0x0F, 0xC6, 0xA0, 0xA2, 0x8D }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x7C, 0xA1, 0x10, 0x45, 0x4A, 0x1A, 0x6E, 0x57 },
      0, { 0 },
      1,
      8, { 0x01, 0xA1, 0xD6, 0xD0, 0x39, 0x77, 0x67, 0x42 },
      8, { 0x59, 0xC6, 0x82, 0x45, 0xEB, 0x05, 0x28, 0x2B }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x31, 0xD9, 0x61, 0x9D, 0xC1, 0x37, 0x6E },
      0, { 0 },
      1,
      8, { 0x5C, 0xD5, 0x4C, 0xA8, 0x3D, 0xEF, 0x57, 0xDA },
      8, { 0xB1, 0xB8, 0xCC, 0x0B, 0x25, 0x0F, 0x09, 0xA0 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x07, 0xA1, 0x13, 0x3E, 0x4A, 0x0B, 0x26, 0x86 },
      0, { 0 },
      1,
      8, { 0x02, 0x48, 0xD4, 0x38, 0x06, 0xF6, 0x71, 0x72 },
      8, { 0x17, 0x30, 0xE5, 0x77, 0x8B, 0xEA, 0x1D, 0xA4 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x38, 0x49, 0x67, 0x4C, 0x26, 0x02, 0x31, 0x9E },
      0, { 0 },
      1,
      8, { 0x51, 0x45, 0x4B, 0x58, 0x2D, 0xDF, 0x44, 0x0A },
      8, { 0xA2, 0x5E, 0x78, 0x56, 0xCF, 0x26, 0x51, 0xEB }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x04, 0xB9, 0x15, 0xBA, 0x43, 0xFE, 0xB5, 0xB6 },
      0, { 0 },
      1,
      8, { 0x42, 0xFD, 0x44, 0x30, 0x59, 0x57, 0x7F, 0xA2 },
      8, { 0x35, 0x38, 0x82, 0xB1, 0x09, 0xCE, 0x8F, 0x1A }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x13, 0xB9, 0x70, 0xFD, 0x34, 0xF2, 0xCE },
      0, { 0 },
      1,
      8, { 0x05, 0x9B, 0x5E, 0x08, 0x51, 0xCF, 0x14, 0x3A },
      8, { 0x48, 0xF4, 0xD0, 0x88, 0x4C, 0x37, 0x99, 0x18 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x70, 0xF1, 0x75, 0x46, 0x8F, 0xB5, 0xE6 },
      0, { 0 },
      1,
      8, { 0x07, 0x56, 0xD8, 0xE0, 0x77, 0x47, 0x61, 0xD2 },
      8, { 0x43, 0x21, 0x93, 0xB7, 0x89, 0x51, 0xFC, 0x98 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x43, 0x29, 0x7F, 0xAD, 0x38, 0xE3, 0x73, 0xFE },
      0, { 0 },
      1,
      8, { 0x76, 0x25, 0x14, 0xB8, 0x29, 0xBF, 0x48, 0x6A },
      8, { 0x13, 0xF0, 0x41, 0x54, 0xD6, 0x9D, 0x1A, 0xE5 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x07, 0xA7, 0x13, 0x70, 0x45, 0xDA, 0x2A, 0x16 },
      0, { 0 },
      1,
      8, { 0x3B, 0xDD, 0x11, 0x90, 0x49, 0x37, 0x28, 0x02 },
      8, { 0x2E, 0xED, 0xDA, 0x93, 0xFF, 0xD3, 0x9C, 0x79 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x04, 0x68, 0x91, 0x04, 0xC2, 0xFD, 0x3B, 0x2F },
      0, { 0 },
      1,
      8, { 0x26, 0x95, 0x5F, 0x68, 0x35, 0xAF, 0x60, 0x9A },
      8, { 0xD8, 0x87, 0xE0, 0x39, 0x3C, 0x2D, 0xA6, 0xE3 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x37, 0xD0, 0x6B, 0xB5, 0x16, 0xCB, 0x75, 0x46 },
      0, { 0 },
      1,
      8, { 0x16, 0x4D, 0x5E, 0x40, 0x4F, 0x27, 0x52, 0x32 },
      8, { 0x5F, 0x99, 0xD0, 0x4F, 0x5B, 0x16, 0x39, 0x69 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x1F, 0x08, 0x26, 0x0D, 0x1A, 0xC2, 0x46, 0x5E },
      0, { 0 },
      1,
      8, { 0x6B, 0x05, 0x6E, 0x18, 0x75, 0x9F, 0x5C, 0xCA },
      8, { 0x4A, 0x05, 0x7A, 0x3B, 0x24, 0xD3, 0x97, 0x7B }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x58, 0x40, 0x23, 0x64, 0x1A, 0xBA, 0x61, 0x76 },
      0, { 0 },
      1,
      8, { 0x00, 0x4B, 0xD6, 0xEF, 0x09, 0x17, 0x60, 0x62 },
      8, { 0x45, 0x20, 0x31, 0xC1, 0xE4, 0xFA, 0xDA, 0x8E }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x02, 0x58, 0x16, 0x16, 0x46, 0x29, 0xB0, 0x07 },
      0, { 0 },
      1,
      8, { 0x48, 0x0D, 0x39, 0x00, 0x6E, 0xE7, 0x62, 0xF2 },
      8, { 0x75, 0x55, 0xAE, 0x39, 0xF5, 0x9B, 0x87, 0xBD }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x49, 0x79, 0x3E, 0xBC, 0x79, 0xB3, 0x25, 0x8F },
      0, { 0 },
      1,
      8, { 0x43, 0x75, 0x40, 0xC8, 0x69, 0x8F, 0x3C, 0xFA },
      8, { 0x53, 0xC5, 0x5F, 0x9C, 0xB4, 0x9F, 0xC0, 0x19 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x4F, 0xB0, 0x5E, 0x15, 0x15, 0xAB, 0x73, 0xA7 },
      0, { 0 },
      1,
      8, { 0x07, 0x2D, 0x43, 0xA0, 0x77, 0x07, 0x52, 0x92 },
      8, { 0x7A, 0x8E, 0x7B, 0xFA, 0x93, 0x7E, 0x89, 0xA3 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x49, 0xE9, 0x5D, 0x6D, 0x4C, 0xA2, 0x29, 0xBF },
      0, { 0 },
      1,
      8, { 0x02, 0xFE, 0x55, 0x77, 0x81, 0x17, 0xF1, 0x2A },
      8, { 0xCF, 0x9C, 0x5D, 0x7A, 0x49, 0x86, 0xAD, 0xB5 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x83, 0x10, 0xDC, 0x40, 0x9B, 0x26, 0xD6 },
      0, { 0 },
      1,
      8, { 0x1D, 0x9D, 0x5C, 0x50, 0x18, 0xF7, 0x28, 0xC2 },
      8, { 0xD1, 0xAB, 0xB2, 0x90, 0x65, 0x8B, 0xC7, 0x78 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x1C, 0x58, 0x7F, 0x1C, 0x13, 0x92, 0x4F, 0xEF },
      0, { 0 },
      1,
      8, { 0x30, 0x55, 0x32, 0x28, 0x6D, 0x6F, 0x29, 0x5A },
      8, { 0x55, 0xCB, 0x37, 0x74, 0xD1, 0x3E, 0xF2, 0x01 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
      0, { 0 },
      1,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      8, { 0xFA, 0x34, 0xEC, 0x48, 0x47, 0xB2, 0x68, 0xB2 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x1F, 0x1F, 0x1F, 0x1F, 0x0E, 0x0E, 0x0E, 0x0E },
      0, { 0 },
      1,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      8, { 0xA7, 0x90, 0x79, 0x51, 0x08, 0xEA, 0x3C, 0xAE }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0xE0, 0xFE, 0xE0, 0xFE, 0xF1, 0xFE, 0xF1, 0xFE },
      0, { 0 },
      1,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      8, { 0xC3, 0x9E, 0x07, 0x2D, 0x9F, 0xAC, 0x63, 0x1D }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      0, { 0 },
      1,
      8, { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF },
      8, { 0x01, 0x49, 0x33, 0xE0, 0xCD, 0xAF, 0xF6, 0xE4 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF },
      0, { 0 },
      1,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      8, { 0xF2, 0x1E, 0x9A, 0x77, 0xB7, 0x1C, 0x49, 0xBC }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF },
      0, { 0 },
      1,
      8, { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
      8, { 0x24, 0x59, 0x46, 0x88, 0x57, 0x54, 0x36, 0x9A }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      0, { 0 },
      1,
      8, { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF },
      8, { 0x6B, 0x5C, 0x5A, 0x9C, 0x5D, 0x9E, 0x0A, 0x5A }
    },

    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      1, { 0xF0 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xF9, 0xAD, 0x59, 0x7C, 0x49, 0xDB, 0x00, 0x5E }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      2, { 0xF0, 0xE1 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xE9, 0x1D, 0x21, 0xC1, 0xD9, 0x61, 0xA6, 0xD6 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      3, { 0xF0, 0xE1, 0xD2 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xE9, 0xC2, 0xB7, 0x0A, 0x1B, 0xC6, 0x5C, 0xF3 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      4, { 0xF0, 0xE1, 0xD2, 0xC3 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xBE, 0x1E, 0x63, 0x94, 0x08, 0x64, 0x0F, 0x05 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      5, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xB3, 0x9E, 0x44, 0x48, 0x1B, 0xDB, 0x1E, 0x6E }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      6, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x94, 0x57, 0xAA, 0x83, 0xB1, 0x92, 0x8C, 0x0D }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      7, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x8B, 0xB7, 0x70, 0x32, 0xF9, 0x60, 0x62, 0x9D }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      8, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xE8, 0x7A, 0x24, 0x4E, 0x2C, 0xC8, 0x5E, 0x82 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      9, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	   0x78 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x15, 0x75, 0x0E, 0x7A, 0x4F, 0x4E, 0xC5, 0x77 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      10, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x12, 0x2B, 0xA7, 0x0B, 0x3A, 0xB6, 0x4A, 0xE0 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      11, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x3A, 0x83, 0x3C, 0x9A, 0xFF, 0xC5, 0x37, 0xF6 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      12, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x94, 0x09, 0xDA, 0x87, 0xA9, 0x0F, 0x6B, 0xF2 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      13, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x88, 0x4F, 0x80, 0x62, 0x50, 0x60, 0xB8, 0xB4 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      14, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x1F, 0x85, 0x03, 0x1C, 0x19, 0xE1, 0x19, 0x68 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      15, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x79, 0xD9, 0x37, 0x3A, 0x71, 0x4C, 0xA3, 0x4F }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      16, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x93, 0x14, 0x28, 0x87, 0xEE, 0x3B, 0xE1, 0x5C }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      17, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x03, 0x42, 0x9E, 0x83, 0x8C, 0xE2, 0xD1, 0x4B }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      18, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xA4, 0x29, 0x9E, 0x27, 0x46, 0x9F, 0xF6, 0x7B }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      19, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11, 0x22 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xAF, 0xD5, 0xAE, 0xD1, 0xC1, 0xBC, 0x96, 0xA8 }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      20, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11, 0x22, 0x33 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x10, 0x85, 0x1C, 0x0E, 0x38, 0x58, 0xDA, 0x9F }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      21, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11, 0x22, 0x33, 0x44 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0xE6, 0xF5, 0x1E, 0xD7, 0x9B, 0x9D, 0xB2, 0x1F }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      22, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11, 0x22, 0x33, 0x44, 0x55 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x64, 0xA6, 0xE1, 0x4A, 0xFD, 0x36, 0xB4, 0x6F }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      23, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x80, 0xC7, 0xD7, 0xD4, 0x5A, 0x54, 0x79, 0xAD }
    },
    {
      ENC_DEC,
      &BLOWFISH, 0, 8,
      24, { 0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87,
	    0x78, 0x69, 0x5A, 0x4B, 0x3C, 0x2D, 0x1E, 0x0F,
	    0x00, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77 },
      0, { 0 },
      1,
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      8, { 0x05, 0x04, 0x4B, 0x62, 0xFA, 0x52, 0xD0, 0x80 }
    },
    {
      ENC_DEC,
      &BLOWFISH, &CBC, 8,
      16, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF,
	    0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87 },
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      1,
      32, { 0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x20,
	    0x4E, 0x6F, 0x77, 0x20, 0x69, 0x73, 0x20, 0x74,
	    0x68, 0x65, 0x20, 0x74, 0x69, 0x6D, 0x65, 0x20,
	    0x66, 0x6F, 0x72, 0x20, 0x00, 0x00, 0x00, 0x00 },
      32, { 0x6B, 0x77, 0xB4, 0xD6, 0x30, 0x06, 0xDE, 0xE6,
	    0x05, 0xB1, 0x56, 0xE2, 0x74, 0x03, 0x97, 0x93,
	    0x58, 0xDE, 0xB9, 0xE7, 0x15, 0x46, 0x16, 0xD9,
	    0x59, 0xF1, 0x65, 0x2B, 0xD5, 0xFF, 0x92, 0xCC }
    },
    {
      ENC_DEC,
      &BLOWFISH, &CFB64, 8,
      16, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF,
	    0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87 },
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      1,
      32, { 0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x20,
	    0x4E, 0x6F, 0x77, 0x20, 0x69, 0x73, 0x20, 0x74,
	    0x68, 0x65, 0x20, 0x74, 0x69, 0x6D, 0x65, 0x20,
	    0x66, 0x6F, 0x72, 0x20, 0x00, 0x00, 0x00, 0x00 },
      32, { 0xE7, 0x32, 0x14, 0xA2, 0x82, 0x21, 0x39, 0xCA,
	    0xF2, 0x6E, 0xCF, 0x6D, 0x2E, 0xB9, 0xE7, 0x6E,
	    0x3D, 0xA3, 0xDE, 0x04, 0xD1, 0x51, 0x72, 0x00,
	    0x51, 0x9D, 0x57, 0xA6, 0xC3, 0x38, 0x4E, 0xCE }
    },
    {
      ENC_DEC,
      &BLOWFISH, &OFB, 8,
      16, { 0x01, 0x23, 0x45, 0x67, 0x89, 0xAB, 0xCD, 0xEF,
	    0xF0, 0xE1, 0xD2, 0xC3, 0xB4, 0xA5, 0x96, 0x87 },
      8, { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 },
      1,
      29, { 0x37, 0x36, 0x35, 0x34, 0x33, 0x32, 0x31, 0x20,
	    0x4E, 0x6F, 0x77, 0x20, 0x69, 0x73, 0x20, 0x74,
	    0x68, 0x65, 0x20, 0x74, 0x69, 0x6D, 0x65, 0x20,
	    0x66, 0x6F, 0x72, 0x20, 0x00 },
      29, { 0xE7, 0x32, 0x14, 0xA2, 0x82, 0x21, 0x39, 0xCA,
	    0x62, 0xB3, 0x43, 0xCC, 0x5B, 0x65, 0x58, 0x73,
	    0x10, 0xDD, 0x90, 0x8D, 0x0C, 0x24, 0x1B, 0x22,
	    0x63, 0xC2, 0xCF, 0x80, 0xDA }
    },
    {
      NONE
    }
  };

static void blockPrint(FILE *fp, const unsigned char *block, int l,
		       const char *tag) {
	int i;
	fprintf (fp, "%s=", tag);
	for (i = 0; i < l; i++) {
		fprintf (fp, "%02X", block[i]);
	}
	fprintf (fp, "\n");
	fflush (fp);
}

int main()
{
  const struct test_st *t;

  for(t = tests; t->kind != NONE; t++)
    {
      switch(t->kind)
	{
	case ENC_DEC:
	case ENCRYPT:
	case DECRYPT:
	  {
	    const SYM_METHOD *sym = t->sym();
	    const MODE_METHOD *mode = t->mode ? t->mode() : 0;
	    ALGO_CTX *ctx = 0;
	    int i;

	    printf("========================================\n");
	    printf("Testing %s of cipher %s-%d-%s\n",
		   t->kind == ENC_DEC ? "encryption and decryption" :
		   (t->kind == ENCRYPT ? "encryption" : "decryption"),
		   SYM_get_name(sym), t->key_bits,
		   mode ? MODE_get_name(mode) : "ECB");
	    blockPrint(stdout, t->key, t->keylen, "Key");
	    if (t->ivlen)
	      blockPrint(stdout, t->iv, t->ivlen, "IV");
	    for(i = 2; i-- > 0;)
	      {
		int status;
		unsigned char input[128], output[128];
		size_t inputlen, outputlen;
		int iterations;

		if (i)
		  {
		    if (t->kind == DECRYPT)
		      continue;
		  }
		else
		  {
		    if (t->kind == ENCRYPT)
		      continue;
		  }

		if (i)
		  {
		    printf("Encrypt:\n");

		    blockPrint(stdout, t->input, t->inputlen, "Input");
		    blockPrint(stdout, t->output, t->outputlen, "Output");
		    memcpy(output, t->input, t->inputlen);
		    outputlen = t->inputlen;
		  }
		else
		  {
		    printf("Decrypt:\n");

		    blockPrint(stdout, t->output, t->outputlen, "Input");
		    blockPrint(stdout, t->input, t->inputlen, "Output");
		    memcpy(output, t->output, t->outputlen);
		    outputlen = t->outputlen;
		  }

		if (mode)
		  {
		    memcpy(input, output, outputlen);
		    inputlen = outputlen;
		  }		

		ctx = SYM_new(sym, mode);
		status = SYM_init(ctx, t->key, t->keylen, t->iv, t->ivlen);
		for(iterations = t->iterations; iterations-- > 0;)
		  {
		    unsigned char iv_copy[64];
		    size_t iv_len;

		    if (!mode)
		      {
			memcpy(input, output, outputlen);
			inputlen = outputlen;
		      }

		    if (status == ALGO_E_OK)
		      {
			if (i)
			  {
			    if (mode)
			      {
				status = MODE_cpy_ivec(ctx, iv_copy,
						       SYM_get_block_bits(sym)/8,
						       &iv_len);
			      }
			    if (status == ALGO_E_OK)
			      status = SYM_encrypt(ctx, input, output,
						   inputlen);
			    if (mode && status == ALGO_E_OK)
			      {
				memcpy(input, iv_copy, iv_len);
				inputlen = iv_len;
			      }
			  }
			else
			  {
			    status = SYM_decrypt(ctx, input, output, inputlen);
			    if (mode && status == ALGO_E_OK)
			      {
				memcpy(input, output, outputlen);
				inputlen = outputlen;
			      }
			  }
		      }
		    if (status != ALGO_E_OK)
		      break;
		  }
		SYM_free(ctx);

		if (status != ALGO_E_OK)
		  {
		    printf("Got error status: %d\n", status);
		    exit(1);
		  }
		else
		  if (i)
		    {
		      if (memcmp(output, t->output, t->outputlen))
			{
			  fprintf(stderr,"Ciphertext mismatch\n");
			  blockPrint(stderr,output,t->outputlen,"Got");
			  blockPrint(stderr,t->output,t->outputlen,"Expected");
			}
		    }
		  else
		    {
		      if (memcmp(output, t->input, t->inputlen))
			{
			  fprintf(stderr,"Ciphertext mismatch\n");
			  blockPrint(stderr,output,t->inputlen,"Got");
			  blockPrint(stderr,t->input,t->inputlen,"Expected");
			}
		    }
	      }
	  }
	  break;
	default:
	  break;
	}
    }
  return 0;
}
