/* $Id: ecb.c,v 1.2 2002/01/24 23:21:25 levitte Exp $ */

/* ====================================================================
 * Copyright (c) 1999 Levitte Programming.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *      This product includes software developed by Levitte Programming
 *      and its contributors.
 *
 * 4. The names "RLCRYPTO" and "Levitte Programming" or the names of its
 *    contributors must not be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *    For written permission, please contact levitte@lp.se.
 *
 * THIS SOFTWARE IS PROVIDED BY LEVITTE PROGRAMMING AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL LEVITTE
 * PROGRAMMING OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

#include <stdlib.h>
#include <string.h>
#include "../algo_int.h"

struct algorithm_context_st
{
  size_t iv_len;
};

static size_t ecb_ctx_size(void)
{
  return sizeof(ALGO_CTX);
}

static ALGO_CTX *ecb_alloc(void)
{
  return (ALGO_CTX *)malloc(ecb_ctx_size());
}

static void ecb_free(ALGO_CTX *ctx)
{
  free(ctx);
}

static int ecb_init(ALGO_CTX *ctx, const unsigned char *ivec, size_t iveclen)
{
  ctx->iv_len = iveclen;	/* Gives us the block size */
  return ALGO_E_OK;
}

static void ecb_clear(ALGO_CTX *ctx)
{
  memset(ctx, 0, ecb_ctx_size());
}

static int ecb_enc_dec(ALGO_CTX *ctx,
		       const unsigned char *input, unsigned char *output, size_t length,
		       int encrypt,
		       int (*sym_enc_dec)(ALGO_CTX *ctx,
					  const unsigned char *input,
					  unsigned char *output, size_t length,
					  int encrypt),
		       ALGO_CTX *sym_data)
{
  int status;
  register size_t iv_len = ctx->iv_len;

  if (length % iv_len != 0)
    return ALGO_E_NOT_EXACT_BLOCKS;

  while (length)
    {
      status = sym_enc_dec(sym_data, input, output, length, encrypt);
      if (status != ALGO_E_OK)
	break;
      input += iv_len;
      output += iv_len;
      length -= iv_len;
    }
  return status;
}

static int ecb_cpy_ivec(ALGO_CTX *ctx, unsigned char *ivec, size_t ivecsize,
			size_t *iveclen)
{
  *iveclen = 0;
  return ALGO_E_OK;
}

static const MODE_METHOD ecb_method =
  {
    "ECB",
    0,
    1,
    &ecb_ctx_size,
    &ecb_alloc,
    &ecb_free,
    &ecb_init,
    &ecb_clear,
    &ecb_enc_dec,
    &ecb_cpy_ivec
  };

const MODE_METHOD *ECB(void)
{
  return &ecb_method;
}
