! DESCRIP.MMS for BAMSE, a VMS SSH library and client.
!
! $Id: descrip.mms,v 1.7 2002/01/17 08:33:36 levitte Exp $
!
! Copyright (c) 2002, Richard Levitte <richard@levitte.org>
! See the file 000LICENSE.TXT for further info.
!
!------------------------------------------------------------

!------------------------------
! We're using zlib 1.1.3

ZLIB = zlib-1_1_3

!------------------------------
! User macros

SSLINCLUDEDIR = SSLINCLUDE:
SSLLIBDIR = SSLLIB:
SSLLIBRARYNAME = LIBCRYPTO.OLB

CC =		cc/decc/warning=(disable=(PREOPTW)) $(CC_DEBUG)
CFLAGS =	/include=([-.$(ZLIB)],[-.RLCRYPTO]) 'debug_cflags' 'internal_cflags'
LDFLAGS =	'debug_ldflags' 'internal_ldflags'

!------------------------------
! Internal macros.
! Please do not change.
EXE =		bamse.exe
EXEMODULES =	
EXELIB =	bamseexe.olb($(EXEMODULES))
LIBMODULES =
LIB =		libbamse.olb($(LIBMODULES))
!LIBRARIES =	$(EXELIB) $(LIB)
SOURCEDIR =	[-.SRC]

! All the source we distribute, from the top directory point of view
TOPFILES =	*.txt *.com
SRCFILES =	[.src]*.com [.src]*.c [.src]*.h [.src]*.cld -
		[.src]*.msg [.src]*.mms [.src]*.rnh [.src]*.opt -
		[.src.include.bamse]*.h [.src.*]*.c [.src.*]*.h [.src.*]*.mms
RLCRYPTOFILES =	[.rlcrypto]*.c [.rlcrypto]*.h [.rlcrypto]*.mms -
		[.rlcrypto.*]*.c [.rlcrypto.*]*.h [.rlcrypto.*]*.mms
UTILFILES =	[.util]*.com [.util]*.mms
DOCFILES =	[.doc]*.* [.doc.html]*.*
ZLIBFILES =	[.$(ZLIB)...]*.*

! Some binaries we also distribute
ZIPDISTEXTRA =	[.bin-*]*.*

! And the OpenSSL library and include files, if they exist.
SSLLIBDIST =	[.ssllib-*]*.*
SSLINCDIST =	[.sslinclude]*.*

! Things not to include in a dist.
DISTEXCEPTIONS = -x wish-*.* bamsemsg.h *.obj

! include version.opt, since that gives us information like version number.
.include version.opt

!------------------------------
! The building part
all : [-]$(EXE) bamse.hlp
	@ !

debug : setup-debug all
	@ !

setup-debug :
	debug_cflags = "/DEBUG/NOOPT"
	debug_ldflags = "/DEBUG/TRACEBACK"
	ZLIB_DEBUG = "/DEBUG"
	RLCRYPTO_DEBUG = "/DEBUG"

make_portable :
	! This is probably paranoid and overly redundant, but that's much
	! better than skiping it all...
	internal_cflags = "/define=(__VMS_VER=60000000,__CRTL_VER=60000000,_VMS_V6_SOURCE=1,_DECC_V4_SOURCE=1,BAMSE_VERSION=""""""" + ($(IDENT) - "V") + """"""")"
	ZLIB_DEFS = "/define=(__VMS_VER=60000000,__CRTL_VER=60000000,_VMS_V6_SOURCE=1,_DECC_V4_SOURCE=1)/warning=disable=PREOPTW"
	RLCRYPTO_DEFS = "/define=(__VMS_VER=60000000,__CRTL_VER=60000000,_VMS_V6_SOURCE=1,_DECC_V4_SOURCE=1)/warning=disable=PREOPTW"

[-]$(EXE) :  $(LIBRARIES) bamse.opt zlib rlcrypto
	set default [-]
	copy [.src]bamse.opt [.BIN-'CPU']
	copy [.src]libbamse.olb [.BIN-'CPU']
	copy [.src]bamseexe.olb [.BIN-'CPU']
	copy [.$(ZLIB)]libz.olb [.BIN-'CPU']
	- link /exec=$(EXE) [.BIN-'CPU']bamseexe/lib /include=(bamse),[.BIN-'CPU']bamse.opt/opt,[.src]version.opt/opt $(LDFLAGS)
	set default [.src]

bamse.hlp : bamse.rnh
	runoff bamse.rnh
	copy bamse.hlp [-]

bamse.opt : descrip.mms
	open/write bamse_opt bamse.opt
	write bamse_opt "[.BIN-''CPU']LIBZ.OLB/LIB"
	write bamse_opt SSLlibrary,"/LIB"
	write bamse_opt "NETLIB_SHRXFR/SHARE"
	if internal_ldflags - ",VAXCRTL.OPT/OPT" .nes. internal_ldflags then -
		write bamse_opt "SYS$SHARE:VAXCRTL/SHARE"
	close bamse_opt

zlib :
	set default [-.$(ZLIB)]
	$(MMS) $(MMS$QUALIFIERS) /MACRO=(__DECC__=1,C_DEB="''ZLIB_DEBUG'",CC_DEFS="''ZLIB_DEFS'") -
		libz.olb
	set default [-.src]

rlcrypto :
	set default [-.rlcrypto]
	$(MMS) $(MMS$QUALIFIERS) /MACRO=(__DECC__=1,C_DEB="''RLCRYPTO_DEBUG'",CC_DEFS="''RLCRYPTO_DEFS'") -
		rlcrypto.olb
	set default [-.src]

bamse_msg.obj : bamse.msg
	message bamse.msg /object=bamse_msg.obj

bamsemsg.h : bamse.msg genmsghdr.com
	@genmsghdr bamse.msg bamsemsg.h

!------------------------------
! Cleaning targets
almostclean :
	- purge/log [-...]
	- delete/log *.obj;*
	- delete/log [-.$(ZLIB)]*.obj;*
	- delete/log [-.RLCRYPTO]*.obj;*
	- delete/log *.exe;*,[-]*.exe;*

olbclean : 
	- delete/log *.olb;*
	- delete/log [-.$(ZLIB)]*.olb;*
	- delete/log [-.RLCRYPTO]*.olb;*

clean : almostclean
	- delete/log [-.BIN-'CPU']bamseexe.olb;*
	- delete/log [-.BIN-'CPU']bamse.olb;*
	- delete/log [-.BIN-'CPU']rlcrypto.olb;*
	- delete/log [-.BIN-'CPU']libz.olb;*

optclean :
	- delete/log bamse.opt;*

distclean : clean optclean olbclean
	@ !

!------------------------------
! distribution targets

! Start with cleaning it all, then do a complete rebuild WITH PORTABILITY
! FLAGS ALL SET, then clean all you don't need, then build the distribution.
! If you think it's rough, well...
dist :  distclean make_portable all almostclean olbclean optclean -
		versions zip.comment copy-openssl
	save_default = f$environment("default")
	set default [-]
	- delete bamse'vmsver'.zip;*
	- open/write z_log zip.log
	- define/user sys$input zip.comment
	- define/user sys$output z_log
	- zip -"Vz" bamse'vmsver'.zip $(TOPFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(SRCFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(RLCRYPTOFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(UTILFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(DOCFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(ZLIBFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(ZIPDISTEXTRA) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(SSLLIBDIST) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(SSLINCDIST) $(DISTEXCEPTIONS)
	- close z_log
	set default 'save_default'

dist-nossllib : distclean make_portable all almostclean olbclean optclean -
		versions zip.comment
	save_default = f$environment("default")
	set default [-]
	- delete bamse'vmsver'.zip;*
	- open/write z_log zip.log
	- define/user sys$input zip.comment
	- define/user sys$output z_log
	- zip -"Vz" bamse'vmsver'.zip $(TOPFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(SRCFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(RLCRYPTOFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(UTILFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(DOCFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(ZLIBFILES) $(DISTEXCEPTIONS)
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'.zip $(ZIPDISTEXTRA) $(DISTEXCEPTIONS)
	- close z_log
	set default 'save_default'

distsrc : versions copy-openssl
	save_default = f$environment("default")
	set default [-]
	- delete bamse'vmsver'_src.zip;*
	- open/write z_log zip.log
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(TOPFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(SRCFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(RLCRYPTOFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(UTILFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(DOCFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(ZLIBFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(SSLLIBDIST) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(SSLINCDIST) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- close z_log
	set default 'save_default'

distsrc-nossllib : versions
	save_default = f$environment("default")
	set default [-]
	- delete bamse'vmsver'_src.zip;*
	- open/write z_log zip.log
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(TOPFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(SRCFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(RLCRYPTOFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(UTILFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(DOCFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- define/user sys$output z_log
	- zip -"V" bamse'vmsver'_src.zip $(ZLIBFILES) $(DISTEXCEPTIONS) *.obj *.olb link.com
	- close z_log
	set default 'save_default'

snapshot : distsrc-nossllib
	today = f$element(0," ",f$cvtime(,"COMPARISON")) - "-" - "-"
	rename [-]bamse'vmsver'_src.zip bamse'vmsver'_src_'today'

versions : version.opt FRC
	@ alpha = f$element(1," ",f$extract(1,20,$(IDENT)))
	@ alpha2 = "0"
	@ if f$length(alpha) .gt. 5 then -
		alpha2 = f$extract(5,f$length(alpha)-5,alpha)
	@ if alpha2 .nes. "alpha" .and. f$length(alpha) .gt. 4 then -
		alpha2 = f$extract(4,f$length(alpha)-4,alpha)
	@ version = f$element(0," ",f$extract(1,10,$(IDENT)))
	@ major = f$element(0,".",version)
	@ minor = f$element(0,"-",f$element(1,".",version))
	@ update = f$element(1,"-",version)
	@ vmsver = "0''major'''minor'"
	@ if major .gt. 9 then vmsver = "''major'''minor'"
	@ if update .nes. "-" then vmsver = "U''update'"+vmsver
	@ if (f$edit(f$extract(0,5,alpha),"lowercase") .eqs. "alpha" -
	      .or f$edit(f$extract(0,4,alpha),"lowercase") .eqs. "beta") -
	     .and. f$type(alpha2) .eqs. "INTEGER" then -
		vmsver = vmsver + "_" + alpha
	@ show symbol alpha*
	@ show symbol vmsver
	
zip.comment : version.opt descrip.mms
	- open/write foo [-]zip.comment
	- write foo ""
	- write foo "BAMSE ''version' -- an SSH library and client for VMS"
	- write foo ""
	- write foo "Runs on both OpenVMS/VAX and OpenVMS/AXP."
	- write foo "Requires NETLIB!  Look for it at ftp://ftp.wku.edu/vms/fileserv/"
	- write foo ""
	- write foo "To get the executable, just do @LINK after you extracted"
	- write foo "the files."
	- write foo ""
	- close foo

copy-openssl :
	if f$parse("[-.SSLLIB-''CPU']") .eqs. "" then -
		create/dir [-.SSLLIB-'CPU']
	copy_files := NO
	copy_from := 'SSLlibrary'
	copy_to := [-.SSLLIB-'CPU']LIBCRYPTO.OLB
	if f$search(copy_to) .eqs. "" then copy_files := YES
	if .not. copy_files then -
		if f$file(copy_from,"FID") .nes. f$file(copy_to,"FID") then -
			copy_files := YES
	if copy_files then copy 'SSLlibrary' [-.SSLLIB-'CPU']LIBCRYPTO.OLB
	if f$parse("[-.SSLINCLUDE]") .eqs. "" then -
		create/dir [-.SSLINCLUDE]
	copy_files := NO
	copy_from := 'SSLinclude'rand.h
	copy_to := [-.SSLINCLUDE]rand.h
	if f$search(copy_to) .eqs. "" then copy_files := YES
	if .not. copy_files then -
		if f$file(copy_from,"FID") .nes. f$file(copy_to,"FID") then -
			copy_files := YES
	if copy_files then copy 'SSLinclude'*.* [-.SSLINCLUDE]
	purge [-.SSL*]

FRC :
	@ ! This clause is to force other clauses to happen.

!------------------------------
! dependencies
depend : bamsemsg.h

! include the common mms file
.include [-.util]common.mms
